#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>


#define US2CLOCK(us) (((uint32_t)(us) * 60398U) >> 16)

/*
 *       
 *    . ,    
 *    ,   1   
 *  (  )
 */

static uint8_t PROGMEM pin_map[32] = {
	7, 3, 2, 6, 5, 1, 0, 4,
	7, 3, 2, 6, 5, 1, 0, 4,
	4, 0, 1, 5, 6, 2, 3, 7,
	4, 0, 1, 5, 6, 2, 3, 7,
};

uint16_t calc_ocr[4][9] = {
	{
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(20500 - 8 * 1500),
	},
	{
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(20500 - 8 * 1500),
	},
	{
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(20500 - 8 * 1500),
	},
	{
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500), US2CLOCK(1500),
		US2CLOCK(20500 - 8 * 1500),
	},
};

uint16_t* table_ptr[4] = {
	calc_ocr[0],
	calc_ocr[1],
	calc_ocr[2],
	calc_ocr[3],
};

void set_position(uint8_t n, uint16_t pos)
{
	if (n > 31) return;
	if (pos < 500) pos = 500;
	else if (pos > 2500) pos = 2500;
	uint8_t idx = pgm_read_byte(pin_map+n);
	uint8_t block = n >> 3;
	pos = US2CLOCK(pos);
	calc_ocr[block][8] += calc_ocr[block][idx];
	calc_ocr[block][8] -= pos;
	calc_ocr[block][idx] = pos;
}

int main(void)
{
	/*        */
	DDRE = _BV(2) | _BV(3) | _BV(4) | _BV(5);
	DDRB = _BV(7);

	/*      */
	PORTE |= _BV(2);
	/*       */
	OCR3A = 2000;
	OCR3B = 2000;
	OCR3C = 2000;
	OCR1C = 2000;

	/*       0  0xFFFF.   
	 *   OCR3X   OC3X   . */
	TCCR3A = _BV(COM3A0) | _BV(COM3B0) | _BV(COM3C0);
	/*    = 8 */
	TCCR3B = _BV(CS31);
	/*       OCRXX */
	ETIMSK = _BV(OCIE3A);
	ETIMSK |= _BV(OCIE3B);
	ETIMSK |= _BV(OCIE3C);

	TCCR1A = _BV(COM1C0);
	/*    = 8 */
	TCCR1B = _BV(CS11);
	/*       OCR1C */
	ETIMSK |= _BV(OCIE1C);

	/*      */
	PORTE &= ~_BV(2);
	sei();					/*   */

	for(;;)  {
		
	}
}

